#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs_1 = require("fs");
const protobuf = __importStar(require("./protobuf"));
const jsgen = __importStar(require("./jsgen"));
const yargs_1 = __importDefault(require("yargs/yargs"));
async function cli() {
    try {
        const argv = await yargs_1.default(process.argv.slice(2))
            .options({
            'proto-file': {
                type: 'string',
                demandOption: true,
                describe: 'Protobuf definition file',
            },
            'output-dir': {
                type: 'string',
                demandOption: true,
                describe: 'Ouput director for generated code'
            },
            'skip-grpc-service': {
                type: 'boolean',
                default: false,
                describe: 'Disable generation of GrpcService (special for server-side)'
            },
        })
            .version()
            .help().alias('h', 'help')
            .argv;
        const rootProtoShortname = path.parse(argv['proto-file']).name;
        const codegenToken = 'codegen';
        /**
         * Загрузка json-схемы с корневого proto-файла. Это нужно для получения
         * списка всех используемых proto-файлов, а также для последующей
         * генерации клиентского GrpcService.
         */
        const packageDefinition = await protobuf.loadPackageDefinition(argv['proto-file']);
        /**
         * Каталог для сгенерированных protobufjs файлов вида app_pb.js и app_pb.d.ts
         * создается по пути, указанному в --output-dir, это может быть, как каталог
         * с серверным .proto файлов, так и клиентский каталог для последущей генерации
         * GrpcService.
         */
        const codegenDirectory = path.resolve(argv['output-dir'], codegenToken);
        await fs_1.promises.mkdir(codegenDirectory, { recursive: true });
        /**
         * Генерация пакетов, сервисов и их объектов с proto-файла,
         * указанного в --proto-file (включая импортированные proto-файлы),
         * в том числе методы кодирования/декодирования Buffer сообщений.
         */
        const staticObjectsSource = await protobuf.generateStaticObjects(packageDefinition.files);
        const staticObjectsFilename = path.resolve(codegenDirectory, `${rootProtoShortname}_pb.js`);
        await fs_1.promises.writeFile(staticObjectsFilename, staticObjectsSource);
        /**
         * Генерация описания типов для уже сгенерированных пакетов,
         * сервисов и объектов proto-файла.
         */
        const staticDeclarationsSource = await protobuf.generateStaticDeclarations(staticObjectsFilename);
        const staticDeclarationsFilename = path.resolve(codegenDirectory, `${rootProtoShortname}_pb.d.ts`);
        await fs_1.promises.writeFile(staticDeclarationsFilename, staticDeclarationsSource);
        /**
         * Генерация клиентсткого GrpcService. Является оберткой
         * над уже сгенерированными объектами grpc. Нужен для упрощения
         * работы с grpc, так как содержит созданные классы всех сервисов
         * на основе промисов.
         */
        if (!argv['skip-grpc-service']) {
            const staticObjectsRelativeFilename = './' + codegenToken + `/${rootProtoShortname}_pb`;
            const grpcServiceSource = jsgen.createGrpcServiceSource(packageDefinition, staticObjectsRelativeFilename);
            const grpcServiceFilename = path.resolve(argv['output-dir'], 'GrpcService.ts');
            await fs_1.promises.writeFile(grpcServiceFilename, grpcServiceSource);
        }
    }
    catch (error) {
        console.error('grpcw-service-generator [error]:', error);
        process.exit(1);
    }
}
cli();
